@@ -1,5 +1,5 @@
 ;; args.scm -- Command line argument handling.
-;; Copyright (C) 2013, 2016 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2016, 2018 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;;
 ;; This file is part of the GNU Shepherd.
@@ -133,13 +133,13 @@
   (set! options
 	(cons* (make <option>
 		 #:long "version"
-		 #:description "display version information and exit"
+		 #:description (l10n "display version information and exit")
 		 #:action (lambda ()
 			    (display-version program-name)
 			    (quit)))
 	       (make <option>
 		 #:long "usage"
-		 #:description "display short usage message and exit"
+		 #:description (l10n "display short usage message and exit")
 		 #:action (lambda ()
 			    (display program-name)
 			    (display " ")
@@ -186,24 +186,34 @@
 			    (quit)))
 	       (make <option>
 		 #:long "help"
-		 #:description "display this help and exit"
+		 #:description (l10n "display this help and exit")
 		 #:action (lambda ()
 			    (for-each display
 				      (list program-name
-					    " [OPTIONS...] "
+					    (l10n " [OPTIONS...] ")
 					    args-syntax))
 			    (newline)
 			    (display args-desc)
 			    (newline)
 			    (for-each display-doc (reverse! options))
-			    (display (string-append "
+
+                            ;; TRANSLATORS: The '~a' placeholders indicate the
+                            ;; bug-reporting address, the name of this
+                            ;; package, and its home page URL.  Please add
+                            ;; another line saying "Report translation bugs to
+                            ;; ...\n" with the address for translation bugs
+                            ;; (typically your translation team's web or email
+                            ;; address).
+			    (format #t (l10n "
 Mandatory or optional arguments to long options are also mandatory or
 optional to the corresponding short options.
 
-Report bugs to: " bug-address ".
-" package-name " general home page: <" package-url ">
-General help using GNU software: <http://www.gnu.org/gethelp/>
-"))
+Report bugs to: ~a .
+~a general home page: <~a>
+General help using GNU software: <http://www.gnu.org/gethelp/>~%")
+                                    bug-address
+                                    package-name
+                                    package-url)
 			    (quit)))
 	       options))
   (let next-arg ((ptr args))
@@ -244,7 +254,7 @@ General help using GNU software: <http://www.gnu.org/gethelp/>
 			     (l10n "Option `--~a' is ambigous.")
 			   (l10n "Unknown option: `--~a'."))
 			 name)
-			(local-output "Try `--help'.")
+			(local-output (l10n "Try `--help'."))
 			(quit 1)))
 		 (and (takes-arg? target-option)
 		      (not (optional-arg? target-option))
@@ -268,7 +278,8 @@ General help using GNU software: <http://www.gnu.org/gethelp/>
 		    (param #f))
 	       (if (not target-option)
 		   (begin
-		     (local-output "Unknown option: `-~a'." opt-char)
+		     (local-output (l10n "Unknown option: `-~a'.")
+                                   opt-char)
 		     (quit 1))
 		 (if (takes-arg? target-option)
 		     (begin
@@ -280,7 +291,8 @@ General help using GNU software: <http://www.gnu.org/gethelp/>
 			       (or (optional-arg? target-option)
 				   (begin
 				     (local-output
-				      "Argument required by `-~a'." opt-char)
+				      (l10n "Argument required by `-~a'.")
+                                      opt-char)
 				     (quit 1)))
 			     (begin
 			       (set! ptr (cdr ptr))
