@@ -1,5 +1,5 @@
 ;; herd.scm -- The program to herd the Shepherd.
-;; Copyright (C) 2013, 2014, 2016 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2014, 2016, 2018 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;;
 ;; This file is part of the GNU Shepherd.
@@ -46,6 +46,17 @@ of pairs."
 
 (define (display-status-summary services)
   "Display a summary of the status of all of SERVICES."
+  (define (service<? service1 service2)
+    (string<? (symbol->string (service-canonical-name service1))
+              (symbol->string (service-canonical-name service2))))
+
+  (define (display-services header bullet services)
+    (unless (null? services)
+      (display header)
+      (for-each (lambda (service)
+                  (format #t " ~a ~a~%" bullet
+                          (service-canonical-name service)))
+                (sort services service<?))))      ;get deterministic output
   (call-with-values
       (lambda ()
         (partition (match-lambda
@@ -53,10 +64,10 @@ of pairs."
                       (car (assoc-ref properties 'running))))
                    services))
     (lambda (started stopped)
-      (format #t (l10n "Started: ~a~%")
-              (map service-canonical-name started))
-      (format #t (l10n "Stopped: ~a~%")
-              (map service-canonical-name stopped)))))
+      (display-services (l10n "Started:\n") "+"
+                        started)
+      (display-services (l10n "Stopped:\n") "-"
+                        stopped))))
 
 (define (display-detailed-status services)
   "Display the detailed status of SERVICES."
@@ -161,23 +172,23 @@ talking to shepherd"))
 
 ;; Main program.
 (define (main . args)
-  (false-if-exception (setlocale LC_ALL ""))
+  (initialize-cli)
 
   (parameterize ((program-name "herd"))
     (let ((socket-file default-socket-file)
           (command-args '()))
       (process-args (program-name) args
-                    "ACTION SERVICE [ARG...]"
-                    (string-append
-                     "Apply ACTION (start, stop, status, etc.) on SERVICE"
-                     " with the ARGs.")
+                    (l10n "ACTION SERVICE [ARG...]")
+                    (l10n "Apply ACTION (start, stop, status, etc.) on \\
+SERVICE with the ARGs.")
                     (lambda (arg)
                       ;; Collect unknown args.
                       (set! command-args (cons arg command-args)))
                     (make <option>
                       #:long "socket" #:short #\s
-                      #:takes-arg? #t #:optional-arg? #f #:arg-name "FILE"
-                      #:description "send commands to FILE"
+                      #:takes-arg? #t #:optional-arg? #f
+                      #:arg-name (l10n "FILE")
+                      #:description (l10n "send commands to FILE")
                       #:action (lambda (file)
                                  (set! socket-file file))))
 
