@@ -1,8 +1,9 @@
 ;; service.scm -- Representation of services.
-;; Copyright (C) 2013, 2014, 2015, 2016 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2014, 2015, 2016, 2017, 2018 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Järling <wolfgang@pro-linux.de>
 ;; Copyright (C) 2014 Alex Sassmannshausen <alex.sassmannshausen@gmail.com>
 ;; Copyright (C) 2016 Alex Kost <alezost@gmail.com>
+;; Copyright (C) 2018 Carlo Zancanaro <carlo@zancanaro.id.au>
 ;;
 ;; This file is part of the GNU Shepherd.
 ;;
@@ -64,6 +65,7 @@
             for-each-service
             lookup-services
             respawn-service
+            handle-SIGCHLD
             register-services
             provided-by
             required-by
@@ -77,6 +79,7 @@
             make-system-destructor
             make-init.d-service
 
+            check-for-dead-services
             root-service
             make-actions
 
@@ -100,6 +103,9 @@
 
             condition->sexp))
 
+;; Keep track of lazy initialization of SIGCHLD handler
+(define %sigchld-handler-installed? #f)
+
 ;; Type of service actions.
 (define-record-type <action>
   (make-action name proc doc)
@@ -388,25 +394,36 @@ wire."
 
 ;; Call action THE-ACTION with ARGS.
 (define-method (action (obj <service>) the-action . args)
-  (define (default-action running . args)
+  (define default-action
     ;; All actions which are handled here might be called even if the
     ;; service is not running, so they have to take this into account.
     (case the-action
       ;; Restarting is done in the obvious way.
       ((restart)
-       (if running
-	   (stop obj)
-           (local-output "~a was not running." (canonical-name obj)))
-       (start obj))
+       (lambda (running . args)
+         (if running
+             (stop obj)
+             (local-output "~a was not running." (canonical-name obj)))
+         (apply start obj args)))
       ((status)
        ;; Return the service itself.  It is automatically converted to an sexp
        ;; via 'result->sexp' and sent to the client.
-       obj)
+       (lambda (_) obj))
+      ((enable)
+       (lambda (_)
+         (enable obj)))
+      ((disable)
+       (lambda (_)
+         (disable obj)))
+      ((doc)
+       (lambda (_ . args)
+         (apply doc obj args)))
       (else
-       ;; FIXME: Unknown service.
-       (raise (condition (&unknown-action-error
-                          (service obj)
-                          (action the-action)))))))
+       (lambda _
+         ;; FIXME: Unknown service.
+         (raise (condition (&unknown-action-error
+                            (service obj)
+                            (action the-action))))))))
 
   (let ((proc (or (and=> (lookup-action obj the-action)
                          action-procedure)
@@ -416,21 +433,23 @@ wire."
     ;; information.
     ;; FIXME: Why should the user-implementations not be allowed to be
     ;; called this way?
-    (cond ((eq? proc default-action)
-           (apply default-action (slot-ref obj 'running) args))
-          ((not (running? obj))
-           (local-output "Service ~a is not running." (canonical-name obj))
-           #f)
-          (else
-           (catch #t
-             (lambda ()
-               (apply proc (slot-ref obj 'running) args))
-             (lambda (key . args)
-               ;; Special case: 'root' may quit.
-               (and (eq? root-service obj)
-                    (eq? key 'quit)
-                    (apply quit args))
-               (report-exception the-action obj key args)))))))
+    (catch #t
+      (lambda ()
+        (cond ((eq? proc default-action)
+               (apply default-action obj args))
+              ((not (running? obj))
+               (local-output "Service ~a is not running." (canonical-name obj))
+               #f)
+              (else
+               (apply proc (slot-ref obj 'running) args))))
+      (lambda (key . args)
+        ;; Special case: 'root' may quit.
+        (and (eq? root-service obj)
+             (eq? key 'quit)
+             (apply quit args))
+        (if (eq? key 'srfi-34)
+            (apply throw key args)                ;handled by callers
+            (report-exception the-action obj key args))))))
 
 ;; Display documentation about the service.
 (define-method (doc (obj <service>) . args)
@@ -567,16 +586,8 @@ results."
 		   (defines-action? unknown 'action))
 	      (apply action unknown 'action the-action args)
               (raise (condition (&missing-service-error (name obj))))))
-        (map (lambda (s)
-               (apply (case the-action
-                        ((enable) enable)
-                        ((disable) disable)
-                        ((doc) doc)
-                        (else
-                         (lambda (s . further-args)
-                           (apply action s the-action further-args))))
-                      s
-                      args))
+        (map (lambda (service)
+               (apply action service the-action args))
              which-services))))
 
 ;; EINTR-safe versions of 'system' and 'system*'.
@@ -662,8 +673,8 @@ set when starting a service."
 
 (define* (read-pid-file file #:key (max-delay 5))
   "Wait for MAX-DELAY seconds for FILE to show up, and read its content as a
-number.  Return #f if FILE does not contain a number; otherwise return the
-number that was read (a PID)."
+number.  Return #f if FILE was not created or does not contain a number;
+otherwise return the number that was read (a PID)."
   (define start (current-time))
   (let loop ()
     (catch 'system-error
@@ -673,27 +684,28 @@ number that was read (a PID)."
           (call-with-input-file file get-string-all))))
       (lambda args
         (let ((errno (system-error-errno args)))
-          (if (and (= ENOENT errno)
-                   (< (current-time) (+ start max-delay)))
-              (begin
-                ;; FILE does not exist yet, so wait and try again.
-                ;; XXX: Ideally we would yield to the main event loop
-                ;; and/or use inotify.
-                (sleep 1)
-                (loop))
+          (if (= ENOENT errno)
+              (and (< (current-time) (+ start max-delay))
+                   (begin
+                     ;; FILE does not exist yet, so wait and try again.
+                     ;; XXX: Ideally we would yield to the main event loop
+                     ;; and/or use inotify.
+                     (sleep 1)
+                     (loop)))
               (apply throw args)))))))
 
 (define* (exec-command command
                        #:key
                        (user #f)
                        (group #f)
+                       (log-file #f)
                        (directory (default-service-directory))
                        (environment-variables (default-environment-variables)))
   "Run COMMAND as the current process from DIRECTORY, and with
 ENVIRONMENT-VARIABLES (a list of strings like \"PATH=/bin\".)  File
-descriptors 1 and 2 are kept as is, whereas file descriptor
-0 (standard input) points to /dev/null; all other file descriptors are
-closed prior to yielding control to COMMAND.
+descriptors 1 and 2 are kept as is or redirected to LOG-FILE if it's true,
+whereas file descriptor 0 (standard input) points to /dev/null; all other file
+descriptors are closed prior to yielding control to COMMAND.
 
 By default, COMMAND is run as the current user.  If the USER keyword
 argument is present and not false, change to USER immediately before
@@ -712,16 +724,26 @@ false."
 
      ;; Close all the file descriptors except stdout and stderr.
      (let ((max-fd (max-file-descriptors)))
-       (catch-system-error (close-fdes 0))
 
+       ;; Redirect stdin to use /dev/null
+       (catch-system-error (close-fdes 0))
        ;; Make sure file descriptor zero is used, so we don't end up reusing
        ;; it for something unrelated, which can confuse some packages.
        (dup2 (open-fdes "/dev/null" O_RDONLY) 0)
 
-       (let loop ((i 3))
-         (when (< i max-fd)
-           (catch-system-error (close-fdes i))
-           (loop (+ i 1)))))
+       (when log-file
+         (catch #t
+           (lambda ()
+             ;; Redirect stout and stderr to use LOG-FILE.
+             (catch-system-error (close-fdes 1))
+             (catch-system-error (close-fdes 2))
+             (dup2 (open-fdes log-file (logior O_CREAT O_WRONLY O_APPEND)) 1)
+             (dup2 1 2))
+           (lambda (key . args)
+             (format (current-error-port)
+                     "failed to open log-file ~s:~%" log-file)
+             (print-exception (current-error-port) #f key args)
+             (primitive-exit 1))))
 
      ;; setgid must be done *before* setuid, otherwise the user will
      ;; likely no longer have permissions to setgid.
@@ -747,6 +769,22 @@ false."
            (print-exception (current-error-port) #f key args)
            (primitive-exit 1))))
 
+     ;; As the last action, close file descriptors.  Doing it last makes
+     ;; "error in the finalization thread: Bad file descriptor" issues
+     ;; unlikely on 2.2.
+     (let loop ((i 3))
+       (when (< i max-fd)
+         ;; First try to close any ports associated with file descriptor I.
+         ;; Otherwise the finalization thread might get around to closing
+         ;; those ports eventually, which will raise an EBADF exception (on
+         ;; 2.2), leading to messages like "error in the finalization
+         ;; thread: Bad file descriptor".
+         (for-each (lambda (port)
+                     (catch-system-error (close-port port)))
+                   (fdes->ports i))
+         (catch-system-error (close-fdes i))
+         (loop (+ i 1)))))
+
      (catch 'system-error
        (lambda ()
          (apply execlp program program args))
@@ -760,20 +798,30 @@ false."
                             #:key
                             (user #f)
                             (group #f)
+                            (log-file #f)
                             (directory (default-service-directory))
                             (environment-variables
                              (default-environment-variables)))
   "Spawn a process that executed COMMAND as per 'exec-command', and return
 its PID."
+  ;; Install the SIGCHLD handler if this is the first fork+exec-command call
+  (unless %sigchld-handler-installed?
+    (sigaction SIGCHLD handle-SIGCHLD SA_NOCLDSTOP)
+    (set! %sigchld-handler-installed? #t))
   (let ((pid (primitive-fork)))
     (if (zero? pid)
         (exec-command command
                       #:user user
                       #:group group
+                      #:log-file log-file
                       #:directory directory
                       #:environment-variables environment-variables)
         pid)))
 
+(define %pid-file-timeout
+  ;; Maximum number of seconds we wait for a PID file to show up.
+  5)
+
 (define make-forkexec-constructor
   (let ((warn-deprecated-form
          ;; Until 0.1, this procedure took a rest list.
@@ -792,35 +840,50 @@ the procedure will be the PID of the child process.
 
 When @var{pid-file} is true, it must be the name of a PID file associated with
 the process being launched; the return value is the PID read from that file,
-once that file has been created."
+once that file has been created.  If @var{pid-file} does not show up in less
+than @var{pid-file-timeout} seconds, the service is considered as failing to
+start."
      ((command #:key
                (user #f)
                (group #f)
                (directory (default-service-directory))
                (environment-variables (default-environment-variables))
-               (pid-file #f))
+               (pid-file #f)
+               (pid-file-timeout %pid-file-timeout)
+               (log-file #f))
       (let ((command (if (string? command)
                          (begin
                            (warn-deprecated-form)
                            (list command))
                          command)))
         (lambda args
-          (when pid-file
-            (catch 'system-error
-              (lambda ()
-                (delete-file pid-file))
-              (lambda args
-                (unless (= ENOENT (system-error-errno args))
-                  (apply throw args)))))
+          (define (clean-up file)
+            (when file
+              (catch 'system-error
+                (lambda ()
+                  (delete-file file))
+                (lambda args
+                  (unless (= ENOENT (system-error-errno args))
+                    (apply throw args))))))
+
+          (clean-up pid-file)
+          (clean-up log-file)
 
           (let ((pid (fork+exec-command command
                                         #:user user
                                         #:group group
+                                        #:log-file log-file
                                         #:directory directory
                                         #:environment-variables
                                         environment-variables)))
             (if pid-file
-                (read-pid-file pid-file)
+                (match (read-pid-file pid-file
+                                      #:max-delay pid-file-timeout)
+                  (#f
+                   (catch-system-error (kill pid SIGTERM))
+                   #f)
+                  ((? integer? pid)
+                   pid))
                 pid)))))
      ((program . program-args)
       ;; The old form, documented until 0.1 included.
@@ -892,7 +955,8 @@ Return #f if service is not found."
                  %services))
 
 (define (service-list)
-  "Return the list of services currently defined."
+  "Return the list of services currently defined.  Note: The order of the list
+returned in unspecified."
   (hash-fold (lambda (name services result)
                (let ((service (lookup-canonical-service name services)))
                  (if service
@@ -933,7 +997,7 @@ child left."
                           what (strerror errno))
             '(0 . #f)))))))
 
-(define (respawn-service signum)
+(define (handle-SIGCHLD signum)
   "Handle SIGCHLD, possibly by respawning the service that just died, or
 otherwise by updating its state."
   (let loop ()
@@ -952,40 +1016,43 @@ otherwise by updating its state."
          ;; SERV can be #f for instance when this code runs just after a
          ;; service's 'stop' method killed its process and completed.
          (when serv
-           (slot-set! serv 'running #f)
-           (if (and (respawn? serv)
-                    (not (respawn-limit-hit? (slot-ref serv 'last-respawns)
-                                             (car respawn-limit)
-                                             (cdr respawn-limit))))
-               (if (not (slot-ref serv 'waiting-for-termination?))
-                   (begin
-                     ;; Everything is okay, start it.
-                     (local-output "Respawning ~a."
-                                   (canonical-name serv))
-                     (slot-set! serv 'last-respawns
-                                (cons (current-time)
-                                      (slot-ref serv 'last-respawns)))
-                     (start serv))
-                   ;; We have just been waiting for the
-                   ;; termination.  The `running' slot has already
-                   ;; been set to `#f' by `stop'.
-                   (begin
-                     (local-output "Service ~a terminated."
-                                   (canonical-name serv))
-                     (slot-set! serv 'waiting-for-termination? #f)))
-               (begin
-                 (local-output "Service ~a has been disabled."
-                               (canonical-name serv))
-                 (when (respawn? serv)
-                   (local-output "  (Respawning too fast.)"))
-                 (slot-set! serv 'enabled? #f))))
+           (respawn-service serv))
 
          ;; As noted in libc's manual (info "(libc) Process Completion"),
          ;; loop so we don't miss any terminated child process.
          (loop))))))
 
-;; Install it as the handler.
-(sigaction SIGCHLD respawn-service SA_NOCLDSTOP)
+(define (respawn-service serv)
+  "Respawn a service that has stopped running unexpectedly. If we have
+attempted to respawn the service a number of times already and it keeps dying,
+then disable it."
+  (slot-set! serv 'running #f)
+  (if (and (respawn? serv)
+           (not (respawn-limit-hit? (slot-ref serv 'last-respawns)
+                                    (car respawn-limit)
+                                    (cdr respawn-limit))))
+      (if (not (slot-ref serv 'waiting-for-termination?))
+          (begin
+            ;; Everything is okay, start it.
+            (local-output "Respawning ~a."
+                          (canonical-name serv))
+            (slot-set! serv 'last-respawns
+                       (cons (current-time)
+                             (slot-ref serv 'last-respawns)))
+            (start serv))
+          ;; We have just been waiting for the
+          ;; termination.  The `running' slot has already
+          ;; been set to `#f' by `stop'.
+          (begin
+            (local-output "Service ~a terminated."
+                          (canonical-name serv))
+            (slot-set! serv 'waiting-for-termination? #f)))
+      (begin
+        (local-output "Service ~a has been disabled."
+                      (canonical-name serv))
+        (when (respawn? serv)
+          (local-output "  (Respawning too fast.)"))
+        (slot-set! serv 'enabled? #f))))
 
 ;; Add NEW-SERVICES to the list of known services.
 (define (register-services . new-services)
@@ -1116,6 +1183,21 @@ file when persistence is enabled."
         (lambda (p)
           (format p "~{~a ~}~%" running-services))))))
 
+(define (check-for-dead-services)
+  "Poll each process that we expect to be running, and respawn any which have
+unexpectedly stopped running. This procedure is used as a fallback on systems
+where prctl/PR_SET_CHILD_SUBREAPER is unsupported."
+  (define (process-exists? pid)
+    (catch #t
+      (lambda () (kill pid 0) #t)
+      (lambda _ #f)))
+  (for-each-service (lambda (service)
+                      (let ((running (slot-ref service 'running)))
+                        (when (and (integer? running)
+                                   (not (process-exists? running)))
+                            (local-output "PID ~a (~a) is dead!" running (canonical-name service))
+                            (respawn-service service))))))
+
 (define root-service
   (make <service>
     #:docstring "The root service is used to operate on shepherd itself."
@@ -1219,7 +1301,9 @@ we want to receive these signals."
            (local-output "Running as PID 1, so not daemonizing."))
           (else
            (if (zero? (primitive-fork))
-               #t
+               (begin
+                 (catch-system-error (prctl PR_SET_CHILD_SUBREAPER 1))
+                 #t)
                (primitive-exit 0))))))
      (persistency
       "Safe the current state of running and non-running services.
