@@ -1,8 +1,9 @@
 ;; support.scm -- Various support facilities, used by herd and shepherd.
 ;; Copyright (C) 2014 A.Sassmannshausen <alex.sassmannshausen@gmail.com>
-;; Copyright (C) 2013, 2014, 2016 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2014, 2016, 2018 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;; Copyright (C) 2016 Mathieu Lirzin <mthl@gnu.org>
+;; Copyright (C) 2018 Danny Milosavljevic <dannym@scratchpost.org>
 ;;
 ;; This file is part of the GNU Shepherd.
 ;;
@@ -34,6 +35,9 @@
             mkdir-p
             with-directory-excursion
 
+            initialize-cli
+
+            %gettext-domain
             l10n
             local-output
             display-version
@@ -42,7 +46,8 @@
             display-line
 
             user-homedir
-            default-logfile
+            user-default-log-file
+            default-logfile-date-format
             default-config-file
             default-socket-dir
             default-socket-file
@@ -172,7 +177,12 @@ output port, and PROC's result is returned."
                  (mkdir path))
              (loop tail path))
            (lambda args
-             (if (= EEXIST (system-error-errno args))
+             ;; On GNU/Hurd we can get EROFS instead of EEXIST here.  Thus, if
+             ;; we get something other than EEXIST, check whether PATH exists.
+             ;; See <https://lists.gnu.org/archive/html/guix-devel/2016-02/msg00049.html>.
+             (if (or (= EEXIST (system-error-errno args))
+                     (let ((st (stat path #f)))
+                       (and st (eq? 'directory (stat:type st)))))
                  (loop tail path)
                  (apply throw args))))))
       (() #t))))
@@ -190,6 +200,29 @@ output port, and PROC's result is returned."
 
 
 
+(define %gettext-domain
+  ;; The gettext message domain.
+  "shepherd")
+
+(define (initialize-cli)
+  "Perform the usual initialization for stand-alone Shepherd commands."
+  ;; By default don't annoy users with deprecation warnings.  In practice,
+  ;; 'define-deprecated' in (ice-9 deprecated) arranges so that those warnings
+  ;; are emitted at expansion-time only, but there are cases where they could
+  ;; slip through, for instance when interpreting code.
+  (unless (getenv "GUILE_WARN_DEPRECATED")
+    (debug-disable 'warn-deprecated))
+
+  ;; In Guile 2.2+, the locale is installed by default.
+  (cond-expand
+    (guile-2.0 (false-if-exception (setlocale LC_ALL "")))
+    (else      #t))
+
+  (bindtextdomain %gettext-domain %localedir)
+  (textdomain %gettext-domain)
+  (setvbuf (current-output-port) _IOLBF)
+  (setvbuf (current-error-port) _IOLBF))
+
 ;; Localized version of STR.
 (define l10n gettext)
 
@@ -201,7 +234,7 @@ output port, and PROC's result is returned."
 
 (define* (display-version #:optional (program-name (program-name)))
   (local-output "~a (~a) ~a" program-name package-name Version)
-  (local-output (l10n "Copyright (C) 2016 the Shepherd authors
+  (local-output (l10n "Copyright (C) 2018 the Shepherd authors
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.")))
@@ -246,6 +279,11 @@ There is NO WARRANTY, to the extent permitted by law.")))
                      (string-append user-homedir "/.config"))
                  "/shepherd"))
 
+(define %user-runtime-dir
+  ;; Default runtime directory if shepherd is run as a normal user.
+  (string-append (or (getenv "XDG_RUNTIME_DIR")
+                     (format #f "/run/user/~s" (getuid)))))
+
 (define (make-bare-init-file target)
   "Return #t if a bare init file was created at TARGET; #f otherwise.
 
@@ -269,11 +307,15 @@ TARGET should be a string representing a filepath + name."
 ""(for-each start '())
 ")))))
 
-;; Logfile.
-(define default-logfile
-  (if (zero? (getuid))
-      (string-append %localstatedir "/log/shepherd.log")
-      (string-append %user-config-dir "/shepherd.log")))
+;; Logging.
+(define (user-default-log-file)
+  "Return the file name of the user's default log file."
+  (mkdir-p %user-config-dir #o700)
+  (string-append %user-config-dir "/shepherd.log"))
+
+(define default-logfile-date-format
+  ;; 'strftime' format string to prefix each entry in the log.
+  "%Y-%m-%d %H:%M:%S ")
 
 ;; Configuration file.
 (define (default-config-file)
@@ -296,7 +338,7 @@ create a template configuration file if non exists."
 (define default-socket-dir
   (if (zero? (getuid))
       %system-socket-dir
-      (string-append %user-config-dir "/run")))
+      (string-append %user-runtime-dir "/shepherd")))
 
 ;; Unix domain socket for receiving commands in shepherd.
 (define default-socket-file
